########################################################################################
#
# Module to configure a serial port 
# Requires Perl 5.001 or higher
# 1.0 DJM/UNAVCO March 1997
# 2.0 DJM / UNAVCO January 1998 (Full POSIX Compliance)
#
########################################################################################

package SERIAL;
@ISA = qw(Exporter);
@EXPORT = qw( open_serial_port drop_dtr open_serial_port_nodtr );

use ERROR;
use POSIX qw(:termios_h :errno_h :fcntl_h);


sub open_serial_port {
  # method to configure the port (takes three parameters)
  # $fname -> the file name of the port you want to open
  # $speed   -> one of the follwing text tokens 1200,4800,9600,19200,38400
  #	defaults to 9600
  # $control -> one of follwing text tokens 8N1,8N2,8E1,8O1,7E1,7O1
  #	defaults to 8N1
  #
  # returns: FILE DESCRIPTOR

  my $DEBUG = 0;
  if ( $DEBUG ) { print STDERR "In open_serial_port\n"; }

  my ( $fname, $speed, $control ) = @_;

  if ( $DEBUG ) { print STDERR "Using $fname, $speed, $control\n"; }

  # open the file
  my $fd_int = POSIX::open( $fname, O_RDWR | O_NOCTTY | O_NONBLOCK );
  $| = 1;
  # return undef if this fails
  if ( !defined($fd_int) ) { 
	print STDERR "Could not open $fname\n";
	return undef;
	}

  if ( $DEBUG ) { print STDERR "Opened $fd_int successfully\n"; }

  # Get a new termios object
  my $myDevice = POSIX::Termios->new;

  if( $DEBUG ) { print STDERR "POSIX fd_int = $fd_int\n"; }

  # Get the current attributes for our device
  $myDevice->getattr($fd_int);


  # change the speed of the device
  SPEED_SWITCH:  {
  if ( $speed =~ /ZERO/ ) {
	$myDevice->setispeed(POSIX::B0);
	$myDevice->setospeed(POSIX::B0);
	last SPEED_SWITCH; }
  if ( $speed =~ /1200/ ) { 
	$myDevice->setispeed(POSIX::B1200);
	$myDevice->setospeed(POSIX::B1200);
	last SPEED_SWITCH; }
  if ( $speed =~ /4800/ ) { 
	$myDevice->setispeed(POSIX::B4800);
	$myDevice->setospeed(POSIX::B4800);
	last SPEED_SWITCH; }
  if ( $speed =~ /9600/ ) { 
	$myDevice->setispeed(POSIX::B9600);
	$myDevice->setospeed(POSIX::B9600);
	last SPEED_SWITCH; }
  if ( $speed =~ /19200/ ) {
	$myDevice->setispeed(POSIX::B19200);
	$myDevice->setospeed(POSIX::B19200);
	last SPEED_SWITCH; }
  if ( $speed =~ /38400/ ) { 
	$myDevice->setispeed(POSIX::B38400);
	$myDevice->setospeed(POSIX::B38400);
	last SPEED_SWITCH; }
  # default to 9600
	$myDevice->setispeed(POSIX::B9600);
	$myDevice->setospeed(POSIX::B9600);
  } # end SPEED_SWITCH

  if( $DEBUG ) { print STDERR "SPEED_SWTICH set.\n"; }

  # change the control characteristics of the device
  my $my_cflag = $myDevice->getcflag;

  CONTROL_SWITCH: {
  # Trimble supports 8N1,8N2,8O1,8E1
  if ( $control =~ /8N1/ ) {
	# change to 8 bits, no parity
	# Disable Parity
	$my_cflag &= ~(POSIX::PARENB);
	# Disable 2 stop bits to get one
	$my_cflag &= ~(POSIX::CSTOPB);
	# Mask out the byte size
	$my_cflag &= ~(POSIX::CSIZE);
	# Set our byte size to eight
	$my_cflag |= POSIX::CS8;
	last CONTROL_SWITCH; }
  if ( $control =~ /8N2/ ) {
	# change to 8 bits, no parity
	$my_cflag &= ~PARENB;
	$my_cflag |= CSTOPB;
	$my_cflag &= ~CSIZE;
	$my_cflag |= CS8;
	last CONTROL_SWITCH; }
  if ( $control =~ /8O1/ ) {
	# change to 8 bits, odd parity
	$my_cflag |= PARENB;
	$my_cflag |= PARODD;
	$my_cflag &= ~CSTOPB;
	$my_cflag &= ~CSIZE;
	$my_cflag |= CS8;
	last CONTROL_SWITCH; }
  if ( $control =~ /8E1/ ) {
	# change to 8 bits, even parity
	$my_cflag |= PARENB;
	$my_cflag &= ~PARODD;
	$my_cflag &= ~CSTOPB;
	$my_cflag &= ~CSIZE;
	$my_cflag |= CS8;
	last CONTROL_SWITCH; }
  if ( $control =~ /7E1/ ) {
	# change to 7 bits, even parity
	$my_cflag |= PARENB;
	$my_cflag &= ~PARODD;
	$my_cflag &= ~CSTOPB;
	$my_cflag &= ~CSIZE;
	$my_cflag |= CS7; 
	last CONTROL_SWITCH; }
  if ( $control =~ /7O1/ ) {
	# change to 7 bits, odd parity
	$my_cflag |= PARENB;
	$my_cflag |= PARODD;
	$my_cflag &= ~CSTOPB;
	$my_cflag &= ~CSIZE;
	$my_cflag |= CS7; 
	last CONTROL_SWITCH; }
  #default to 8N1
	# change to 8 bits, no parity
	# Disable Parity
	$my_cflag &= ~PARENB;
	# Disable 2 stop bits to get one
	$my_cflag &= ~CSTOPB;
	# Mask out the byte size
	$my_cflag &= ~CSIZE;
	# Set our byte size to eight
	$my_cflag |= CS8;
  } # end CONTROL_SWITCH

	$my_cflag |= HUPCL;

  $myDevice->setcflag($my_cflag);


  # Make sure we have raw I/O .......
  my $my_lflag = $myDevice->getlflag;
  $my_lflag &= ~ICANON;
  $my_lflag &= ~ECHO;
  $my_lflag &= ~ISIG;
  $my_lflag &= ~ECHOK;
  $myDevice->setlflag($my_lflag);
  if( $my_lflag != ($myDevice->getlflag) ) { print STDERR "Lflag not raw\n"; }
  my $my_oflag = $myDevice->getoflag;
  $my_oflag &= ~OPOST;
  $myDevice->setoflag($my_oflag);

  # Force the changes ...
  $myDevice->setattr( $fd_int, TCSANOW );

  return $fd_int;

} # End open_serial_port

sub drop_dtr {

  my $fname = shift;

  my $fd = open_serial_port( $fname, "ZERO", "8N1" );

  if( !defined( $fd ) ) { 
	return undef;
  } else {
	POSIX::close($fd);
	return 1;
  }

} # End drop_dtr 

sub open_serial_port_nodtr {

  # method to configure the port (takes three parameters)
  # $fname -> the file name of the port you want to open
  # $speed   -> one of the follwing text tokens 1200,4800,9600,19200,38400
  #     defaults to 9600
  # $control -> one of follwing text tokens 8N1,8N2,8E1,8O1,7E1,7O1
  #     defaults to 8N1
  #
  # returns: FILE DESCRIPTOR

  my $DEBUG = 0;
  if ( $DEBUG ) { print STDERR "In open_serial_port\n"; }

  my ( $fname, $speed, $control ) = @_;

  if ( $DEBUG ) { print STDERR "Using $fname, $speed, $control\n"; }

  # open the file
  my $fd_int = POSIX::open( $fname, O_RDWR | O_NOCTTY | O_NONBLOCK );
  $| = 1;
  # return undef if this fails
  if ( !defined($fd_int) ) {
        print STDERR "Could not open $fname\n";
        return undef;
        }

  if ( $DEBUG ) { print STDERR "Opened $fd_int successfully\n"; }

  # Get a new termios object
  my $myDevice = POSIX::Termios->new;

  if( $DEBUG ) { print STDERR "POSIX fd_int = $fd_int\n"; }

  # Get the current attributes for our device
  $myDevice->getattr($fd_int);

  # change the speed of the device
  SPEED_SWITCH:  {
  if ( $speed =~ /ZERO/ ) {
        $myDevice->setispeed(POSIX::B0);
        $myDevice->setospeed(POSIX::B0);
        last SPEED_SWITCH; }
  if ( $speed =~ /1200/ ) {
        $myDevice->setispeed(POSIX::B1200);
        $myDevice->setospeed(POSIX::B1200);
        last SPEED_SWITCH; }
  if ( $speed =~ /4800/ ) {
        $myDevice->setispeed(POSIX::B4800);
        $myDevice->setospeed(POSIX::B4800);
        last SPEED_SWITCH; }
  if ( $speed =~ /9600/ ) {
        $myDevice->setispeed(POSIX::B9600);
        $myDevice->setospeed(POSIX::B9600);
        last SPEED_SWITCH; }
  if ( $speed =~ /19200/ ) {
        $myDevice->setispeed(POSIX::B19200);
        $myDevice->setospeed(POSIX::B19200);
        last SPEED_SWITCH; }
  if ( $speed =~ /38400/ ) {
        $myDevice->setispeed(POSIX::B38400);
        $myDevice->setospeed(POSIX::B38400);
        last SPEED_SWITCH; }
  # default to 9600
        $myDevice->setispeed(POSIX::B9600);
        $myDevice->setospeed(POSIX::B9600);
  } # end SPEED_SWITCH

  if( $DEBUG ) { print STDERR "SPEED_SWTICH set.\n"; }

  # change the control characteristics of the device
  my $my_cflag = $myDevice->getcflag;

  CONTROL_SWITCH: {
  # Trimble supports 8N1,8N2,8O1,8E1
  if ( $control =~ /8N1/ ) {
        # change to 8 bits, no parity
        # Disable Parity
        $my_cflag &= ~(POSIX::PARENB);
        # Disable 2 stop bits to get one
        $my_cflag &= ~(POSIX::CSTOPB);
        # Mask out the byte size
        $my_cflag &= ~(POSIX::CSIZE);
        # Set our byte size to eight
        $my_cflag |= POSIX::CS8;
        last CONTROL_SWITCH; }
  if ( $control =~ /8N2/ ) {
        # change to 8 bits, no parity
        $my_cflag &= ~PARENB;
        $my_cflag |= CSTOPB;
        $my_cflag &= ~CSIZE;
        $my_cflag |= CS8;
        last CONTROL_SWITCH; }
  if ( $control =~ /8O1/ ) {
        # change to 8 bits, odd parity
        $my_cflag |= PARENB;
        $my_cflag |= PARODD;
        $my_cflag &= ~CSTOPB;
        $my_cflag &= ~CSIZE;
        $my_cflag |= CS8;
        last CONTROL_SWITCH; }
  if ( $control =~ /8E1/ ) {
        # change to 8 bits, even parity
        $my_cflag |= PARENB;
        $my_cflag &= ~PARODD;
        $my_cflag &= ~CSTOPB;
        $my_cflag &= ~CSIZE;
        $my_cflag |= CS8;
        last CONTROL_SWITCH; }
  if ( $control =~ /7E1/ ) {
        # change to 7 bits, even parity
        $my_cflag |= PARENB;
        $my_cflag &= ~PARODD;
        $my_cflag &= ~CSTOPB;
        $my_cflag &= ~CSIZE;
        $my_cflag |= CS7;
        last CONTROL_SWITCH; }
  if ( $control =~ /7O1/ ) {
        # change to 7 bits, odd parity
        $my_cflag |= PARENB;
        $my_cflag |= PARODD;
        $my_cflag &= ~CSTOPB;
        $my_cflag &= ~CSIZE;
        $my_cflag |= CS7;
        last CONTROL_SWITCH; }
  #default to 8N1
        # change to 8 bits, no parity
        # Disable Parity
        $my_cflag &= ~PARENB;
        # Disable 2 stop bits to get one
        $my_cflag &= ~CSTOPB;
        # Mask out the byte size
        $my_cflag &= ~CSIZE;
        # Set our byte size to eight
        $my_cflag |= CS8;
  } # end CONTROL_SWITCH

	$my_cflag &= ~HUPCL;

  $myDevice->setcflag($my_cflag);


  # Make sure we have raw I/O .......
  my $my_lflag = $myDevice->getlflag;
  $my_lflag &= ~ICANON;
  $my_lflag &= ~ECHO;
  $my_lflag &= ~ISIG;
  $my_lflag &= ~ECHOK;
  $myDevice->setlflag($my_lflag);
  if( $my_lflag != ($myDevice->getlflag) ) { print STDERR "Lflag not raw\n"; }
  my $my_oflag = $myDevice->getoflag;
  $my_oflag &= ~OPOST;
  $myDevice->setoflag($my_oflag);

  # Force the changes ...
  $myDevice->setattr( $fd_int, TCSANOW );

  return $fd_int;

} # End open_serial_port_nodtr

