##################### PACKAGE Trimble_RFILE #####################
#
# (Ver 1.7 - Apr. 30 1997) Dave Mencin UNAVCO dmencin@unavco.ucar.edu
# This package is a bunch of wrappers for the Trimble Rfile suite of
# programs. All programs expect the first five tokens to be
#
# $rpath	(The path where the rfile executables are located)
# $dev	(The device which the Trimble is attached to (or the modem))
# $baud	(The baud rate to which the Trimble is set)
# #parity	(The parity in Trimble format ( -P? ))
# $connection (The type of connection used, for most cases this would be 
#		DIRECT as using MODEM here implicitly uses rconn, a better
#		way is to use a DIRECT token and make the connection yourself)
#
# After the connection token a variable number of tokens may follow, all
# must be included.
#
# After these tokens are processed, if connection was MODEM you must supply
# an additional two tokens for rconn
#
# $dial       (A dial string)
# $modem      (The modem config file name)
#
# These supply the basic needs to configure and download a Trimble reciever
#
# Alpha Creation: Apr-2 1997
# Added Comments and dummy $result return: Apr-7 1997
# Added strict ordering across functions: Apr-8 1997
# Fixed $return values: Apr-15 1997
# Redirected STDERR to hide all the Tirmble junk: Apr-15 1997
# Pipe failure now generates a error_f: Apr-15 1997
# Added debug variable to switch STDERR redirection: Apr-15 1997
# Added scale_factor: Apr-30 1997
# Fixed scoping bug (used my inside an if statement): May-2 1997
# Added rconn support: May-2 1997
# Added rdisconn support May-2 1997
# Added runpkr00 support: May-2 1997
# Moved runpkr00 to Trimble_DL (only rfile support here): May-4 1997
# Added get_serial: Jan-7 1998
# Added "--rtsctc" flag for rutil version 2.35 or greater
# will only work with this version or greater: Dec 22 1999
# Added Trimble 5700 functionality: May 20 2001
# Normandeau added rsurvey_comm Nov 15 2002
# Normandeau added R7 support with T01 files on 29 APR 08 

package Trimble_RFILE;
use Exporter ();
@ISA = qw(Exporter);
@EXPORT = qw( get_serial rsurvey_comm  get_model config_station get_filelist get_file purge_file connect disconnect);

use ERROR;

# To turn this on from MAIN set Trimble_RFILE::debug_Trimble_RFILE = "YES"
$debug_Trimble_RFILE = "YES";

#############################################################
sub get_serial {

# get_serial $rpath, $dev, $baud, $parity, $connection [,$dial, $modem ]
#
# This function returns $result, $serial 

 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $connection = shift;
 my $dial;
 my $modem;

 my $serial;
 my $key;

 if ($connection eq "MODEM") {
        $dial = shift;
        $modem = shift;
 }

 #Build command
 my $command = $rpath . "rstatus -D$dev -B$baud -P$parity --rtscts -version";
 if ($connection eq "MODEM") {
        $command = $command . " -C$modem -M$dial";
 }

 error_m("Attempting to get serial number for receiver");
 error_m("Using $command for RFILE");

 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR");
   unless(open(STDERR,">/dev/null") ) {error_w("Unable to redirect STDERR");}
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging get_serial STDERR not redirected"); }

 #send rfile command
 unless( open( OUT, "$command |") ) {
        error_f("Unable to execute RFILE ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered

 while(<OUT>) {
   if (/SERIAL/) {
	($key, $serial) = split(/:/);
        $key =~ s/^\s*(.*?)\s*$/$1/;
        $serial =~ s/^\s*(.*?)\s*$/$1/;
	error_m("Receiver Serial Number is $serial");
   }
 }

 close(OUT);

 my $result = $?; # This should return the exit status of rfile (or wait)

 if( $debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging get_serial STDERR not restored"); }
 return ($result,$serial);
 } #end get_serial 

#############################################################
sub get_model {

# get_model $rpath, $dev, $baud, $parity, $connection [,$dial, $modem ]
#
# This function returns $result, $model 

 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $connection = shift;
 my $dial;
 my $modem;

 my $model;
 my $key;

 if ($connection eq "MODEM") {
        $dial = shift;
        $modem = shift;
 }

 #Build command
 my $command = $rpath . "rstatus -D$dev -B$baud -P$parity --rtscts -rcvrtype";
 if ($connection eq "MODEM") {
        $command = $command . " -C$modem -M$dial";
 }

 error_m("Attempting to get model number for receiver");
 error_m("Using $command for RFILE");

 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR");
   unless(open(STDERR,">/dev/null") ) {error_w("Unable to redirect STDERR");}
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging get_model STDERR not redirected"); }

 #send rfile command
 unless( open( OUT, "$command |") ) {
        error_f("Unable to execute RFILE ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered

 while(<OUT>) {
   if (/RCVRTYPE:/) {
	($key, $model) = split(/:/);
        $key =~ s/^\s*(.*?)\s*$/$1/;
        $model =~ s/^\s*(.*?)\s*$/$1/;
	error_m("Receiver type is $model");
   }
 }

 close(OUT);

 my $result = $?; # This should return the exit status of rfile (or wait)

 if( $debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging get_model STDERR not restored"); }
 return ($result,$model);
 } #end get_model 


#############################################################
sub disconnect {

# connect $rpath, $dev, $baud, $parity, $modem
# returns $result from rfile

 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $modem = shift;

 #build command
 my $command = $rpath . "rdisconn -D$dev -B$baud -P$parity --rtscts -C$modem";
 error_m("Using $command for RFILE");

 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR"); # Save STDERR so we can restore it
   unless(open(STDERR,">/dev/null") ) {error_w("Unable to redirect STDERR");}
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging get_filelist STDERR not redirected"); }

 #send rfile command
 unless( open( OUT, "$command |") ) {
	error_f("Unable to execute RFILE ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered

 while(<OUT>) {
 }
 close(OUT);
 my $result = $?; #This should be the return code from our rfile pipe

 if($debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging get_filelist STDERR not restored"); }

 return $result;

} #end disconnect

#############################################################
sub connect {

# connect $rpath, $dev, $baud, $parity, $dial, $modem
# returns 

 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $dial = shift;
 my $modem = shift;

 #build command
 my $command = $rpath . "rconn -D$dev -B$baud -P$parity -C$modem --rtscts -M$dial";
 error_m("Using $command for RFILE");

 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR"); # Save STDERR so we can restore it
   unless(open(STDERR,">/dev/null") ) {error_w("Unable to redirect STDERR");}
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging get_filelist STDERR not redirected"); }

 #send rfile command
 unless( open( OUT, "$command |") ) {
	error_f("Unable to execute RFILE ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered

 while(<OUT>) {
 }
 close(OUT);
 my $result = $?; #This should be the return code from our rfile pipe

 if($debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging get_filelist STDERR not restored"); }

 return $result;

} #end connect

#############################################################
sub get_filelist {

# get_filelist $rpath, $dev, $baud, $parity, $connection [, $dial, $modem, $scale_factor]
# returns hash of index keyed on index (each index is a hash) and $result
# in the form ($result, %filelist) = get_filelist(.....);

 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $connection = shift;
 my $scale_factor = shift;
 my $model = shift;
 my $dial;
 my $modem;


 if ($connection eq "MODEM") {
	$dial = shift;
	$modem = shift;
 }

 #build command
 my $command = $rpath . "rfile -D$dev -B$baud -P$parity --rtscts -T";
 if ($connection eq "MODEM") {
	$command = $command . " -C$modem -M$dial";

	if ($scale_factor != 0) {
		$command = $command . " --to$scale_factor";
	}
 }

 error_m("Using $command for RFILE");
print STDOUT "##-> RFILE COMMAND $command \n";
 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR"); # Save STDERR so we can restore it
   unless(open(STDERR,">/dev/null") ) {error_w("Unable to redirect STDERR");}
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging get_filelist STDERR not redirected"); }

 #send rfile command
print STDOUT "##-> SENDING $command  to OUT \n";
# TEMP HACK
# open( OUT, "$command |") or die "cannot open $command :$!"; 
 unless( open( OUT, "$command |") ) {
	error_f("Unable to execute RFILE ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered

 # read in rfile command
 
 my @records;
 my %filehash;

if(($model ne "R7") and ($model ne "5700")) {

print STDOUT "##-> FILELIST for a $model  \n";
# if($model ne "5700") {
 		   my @fields = ("date","time","coor","index","name","records","bytes");
 		   while(<OUT>) {
   		   # This is a depend if someone were to randomly enter a timezone ident
   		   if (/UTC/) {
		   	  @records = split;
		   	  $index = 0;
		   	  foreach $f (@fields) {
		   	  $filehash{$records[3]}{$f} = $records[$index++];
			  }

   		  }
		  }
 } # end 5700 if


 else{ # connected to  a 5700 or R7
print STDOUT "##-> R7 or 5700  FILELIST is for a $model  \n";
 my @fields = ("name","date","time","bytes");

 while(<OUT>) {

print STDOUT "##->FILELIST LINE:  $_\n";
   # This is a depend if someone were to randomly enter a timezone ident
   	 if (/T00|T01/)  {
print STDOUT "##->FOUND a file $_\n";
	 @records = split;
	 $index = 0;
	 		foreach $f (@fields) {
			
			$filehash{$records[0]}{$f} = $records[$index++];

			}
  	  }
   }
 } # end 5700 else

 close(OUT);
 my $result = $?; #This should be the return code from our rfile pipe

 if($debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging get_filelist STDERR not restored"); }

 return( $result, %filehash );

} #end get_filelist

#################################################################
sub get_file {

# get_file $rpath, $dev, $baud, $parity, $connection, $index, $filename
#                                        $fast [, $dial, $modem, $scale_factor ]
#
# This function will get a file of a given index and place it in filename
# The fast flag implements Trimble's Fast Download option.

 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $connection = shift;
 my $index = shift;
 my $filename = shift;
 my $fast = shift;
 my $scale_factor = shift;
 my $dial;
 my $modem;

 if ($connection eq "MODEM") {
	$dial = shift;
	$modem = shift;
 }

 #build command
 print STDOUT (" Downloading file $index\n");
 my $command = $rpath . "rfile -D$dev -B$baud -P$parity --rtscts -I$index,$filename";
 if ($connection eq "MODEM") {
	$command = $command . " -C$modem -M$dial";
	if ($scale_factor != 0 ) {
		$command = $command . " --to$scale_factor";
	}
 }
 if ($fast eq "YES") {
	$command = $command . " -F";
 }

error_m("Using $command for RFILE");

 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR");
   unless(open(STDERR,">/dev/null") ) {error_w("Unable to redirect STDERR");}
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging get_file STDERR not redirected"); }

 #send rfile command
 unless( open( OUT, "$command |") ) {
	error_f("Unable to execute RFILE ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered

 close(OUT);

 my $result = $?; # This should return the exit status of rfile (or wait)

 if( $debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging get_file STDERR not restored"); }

 return $result;
 } #end get_file

#################################################################
sub purge_file {

# purge_file $rpath, $dev, $baud, $parity, $connection, $index [,$dial, $modem ]
#   
# This function purges file on index

 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $connection = shift;
 my $index = shift;
 my $dial;
 my $modem;

 if ($connection eq "MODEM") {
	$dial = shift;
	$modem = shift;
 }

 #build command

 my $command = $rpath . "rfile -D$dev -B$baud -P$parity --rtscts -K$index";
 if ($connection eq "MODEM") {
	$command = $command . " -C$modem -M$dial";
 }

print STDOUT "STDOUT issuing $command \n";
 error_m("Attempting to purge file index $index");
 error_m("Using $command for RFILE");

 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR");
   unless(open(STDERR,">/dev/null") ) {error_w("Unable to redirect STDERR");}
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging purge_file STDERR not redirected"); }

 #send rfile command
 unless( open( OUT, "$command |") ) {
	error_f("Unable to execute RFILE ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered

 close(OUT);

 my $result = $?; # This should return the exit status of rfile (or wait)

 if( $debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging purge_file STDERR not restored"); }

 return $result;
 } #end purge_file

#################################################################
sub config_station {

 #Configure a station

 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $connection = shift;
 my $station_id = shift;
 my $sln = shift;
 my $scale_factor = shift;
 my $dial;
 my $modem;

 if ($connection eq "MODEM") {
	$dial = shift;
	$modem = shift;
 }
 
 my $index = 21; #This may be dangerous

 # build the command
 my $r_command = $rpath . "rstation -D$dev --rtscts -I$index -S$station_id -N$sln" ;
 my $port_info = " -B$baud -P$parity";
 my $command = " "; 
 if ($connection eq "MODEM") {
	my $dial_command = " -D$dial -C$modem --to$scale_factor";
	$command = $r_command . $dial_command;
 }
 $command = $command . $port_info;

 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR");
   open(STDERR,">/dev/null") or warn "Unable to redirect STDERR";
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging config_station STDERR not redirected"); }

 #send rfile command
 unless( open( OUT, "$command |") ) {
	error_f("Unable to execute RFILE ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered

 close(OUT);

 my $result = $?; # This should return the exit status of rfile (or wait)

 if( $debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging config_station STDERR not restored"); }

 return $result;

} # end config_station

#############################################################
sub rsurvey_comm {
                                                                                        
# rsurvey_comm ($rpath, $dev, $baud, $parity),  $SITE
#
# This function returns $result
                                                                                        
 my $rpath = shift;
 my $dev = shift;
 my $baud = shift;
 my $parity = shift;
 my $SITE = shift;
                                                                                        
 #Build command
 my $command = $rpath . "rsurvey -D$dev -B$baud -P$parity --rtscts -Q";
 #my $command = $rpath . "rsurvey -D$dev -B$baud -P$parity  -T";
                                                                                        
 error_m("Attempting to restart receiver");
 error_m("Using $command for RSURVEY");
                                                                                        
 if ($debug_Trimble_RFILE eq "NO") {
   #redirect stderr so the user doesn't see all the Trimble crap.
   open(SAVEERR,">&STDERR");
   unless(open(STDERR,">/dev/null") ) {error_w("Unable to redirect STDERR");}
   select(STDERR); $| = 1; #make unbuffered
 } else { error_m("Debugging rsurvey_comm STDERR not redirected"); }
                                                                                        
 #send rfile command
                                                                                        
unless( open( OUT, "$command |") ) {
        error_f("Unable to execute RSURVEY ($command)");
 }
 select(OUT); $| = 1; #make OUT unbuffered
                                                                                        
 while(<OUT>) {
        error_m("$_\n");
 }
                                                                                        
                                                                                        
 close(OUT);
                                                                                        
 my $result = $?; # This should return the exit status of rfile (or wait)
                                                                                        
 if( $debug_Trimble_RFILE eq "NO") {
 #restore STDERR
 close(STDERR);
 open(STDERR,">&SAVEERR");
 } else { error_m("Debugging get_serial STDERR not restored"); }
 return ($result,$serial);
                                                                                        
# send an email to owner to let them know
$subject = "Station  $SITE not logging";
open(SEND, "| /bin/mail -s \"$subject\"  jnormand  ") || die " Mail crapped out $!\n";
        print SEND "$SITE RESULT $result \n";
        print SEND "\n";
        print SEND " \n\n";
close SEND;
                                                                                        
                                                                                        
                                                                                        
 } #end rsurvey_comm
                                                                                        

